/*
 * getsym.c -- A program to get the value of a given symbol in the kernel
 *             namelist.
 *
 * To use, simply type:
 * # getmax <symbol_name>
 * e.g. # getmax maxusers
 * Will show you the value of maxusers.
 *
 * With some modification, this code *could* be used to actually modify
 * kernel params.  However, this would be Bad Mojo unless you REALLY
 * know what you're doing.  You don't, trust me.
 * You MUST be root to run this utility.  That's because you are accessing
 * /dev/ksyms.  If just anyone could access /dev/ksyms, well... ;-)
 *
 * To compile: cc -o getsym -lelf getsym.c
 * 
 */

#ifndef lint
static char sccsid[] = "@(#)getsym.c v1.0 07 AUG 1997 Robert Owen Thomas, robt@cymru.com";
#endif

#include <nlist.h>
#include <stdio.h>
#include <fcntl.h>

int
main(int argc, char *argv[])
{
	int     fd;
	int     maxusers;

	static struct nlist nl[2];

	if (argc != 2) {
		fprintf(stderr, "%s: usage: %s <symbol>\n",argv[0],argv[0]);
		exit(2);
	}

	nl[0].n_name = argv[1];

	/* retrieve symbols from the kernel */
	if (nlist("/dev/ksyms", nl) == -1) {
		perror("nlist");
		exit(1);
	}

	/* open kernel memory, read-only */
	fd = open("/dev/kmem", O_RDONLY);
	if (fd == -1) {
		perror("open");
		exit(1);
	}

	/* seek to the specified kernel address */
	if (lseek(fd, nl[0].n_value, 0) == -1) {
		perror("lseek");
		exit(1);
	}

       /* read the value from kernel memory */
	if (read(fd, &maxusers, sizeof maxusers) == -1) {
		perror("read");
		exit(1);
	}

	/* print and done */
	printf("%s(0x%x) = %d\n", argv[1], nl[0].n_value, maxusers);
	close(fd);
	exit(0);
}

